//////////////////////////////////////////////////////////////////////////////

font alert_font = <alert.pcx>, 34, 38;

//////////////////////////////////////////////////////////////////////////////

var video_mode = 7; // 800x600
var video_depth = 16; // 16 bit mode

var lives = 3; // the player gets 3 lives
var score = 0;
var barrel_offset = 0;
var rocket_coords = 0;
var rocket_speed;
var vehicle_speed;
var speed_offset;
var bomb_speed;

//////////////////////////////////////////////////////////////////////////////

sound beep_sound = <beep.wav>;
sound rocket_wav = <rocket.wav>;
sound explosion_wav = <explosion.wav>;
sound lost_wav = <lost.wav>;
sound jump_wav = <jump.wav>;
sound landed_wav = <landed.wav>;

//////////////////////////////////////////////////////////////////////////////

string alert_wmb = <alert.wmb>;
string rocket_mdl = <rocket.mdl>;
string barrel_mdl = <barrel.mdl>;
string explosion_pcx = <explo+6.pcx>;

//////////////////////////////////////////////////////////////////////////////

function attach_barrel();
function fire_rocket();
function rocket_explodes();
function rocket_explodes();
function animated_explosion();
function vehicle_jumps();
function obstacle_hit();
function destroy_ufo();
function fire_bomb();
function hit_player();

//////////////////////////////////////////////////////////////////////////////

entity* rocket_ptr;

//////////////////////////////////////////////////////////////////////////////

text moon_text 
{
	font = alert_font;
	pos_x = 0;
	pos_y = 0;
	string = "Score:        Lives:";
	flags = visible;
}

panel moon_panel // displays the score using 3 digits and lives using 1 digit
{
	pos_x = 0;
	pos_y = 0;
	digits = 190, 0, 3, alert_font, 1, score; 
	digits = 720, 0, 1, alert_font, 1, lives; 

	flags = refresh, d3d, visible;
}

//////////////////////////////////////////////////////////////////////////////

function main()
{
	clip_size = 0; // show all the triangles for all the models
	fps_max = 30; // no need to have over 30 fps
	level_load (alert_wmb);
}

action players_vehicle
{
	player = me;
	my.z = 20; // place the vehicle 20 quants above the origin
	ent_create(barrel_mdl, nullvector, attach_barrel); 
	vehicle_speed.y = 0;
	vehicle_speed.z = 0;
	while (lives > 0)
	{
		if (key_cul == 1 && speed_offset > 0) // use cursor left to slow down the vehicle
		{
			speed_offset -= 1 * time;
		}
		if (key_cur == 1 && speed_offset < 5) // use cursor up to increase the speed of the vehicle
		{
			speed_offset += 1 * time;
		}

		vehicle_speed.x = 5 + speed_offset; // speed = 5...10
		vehicle_speed *= time;

		if (key_space == 1) 
		{
			vehicle_jumps();
		}

		move_mode = ignore_passable;
		ent_move (vehicle_speed, nullvector);

		camera.x = my.x;
		camera.y = -2500;
		camera.z = 700;
		camera.pan = 90;

		score += 0.01 * time; // add to the score as long as the player manages to survive

		wait (1);
	}
}

function attach_barrel()
{
	proc_late(); // prevent shaking
	my.passable = on;
	while(lives > 0) // as long as the player isn't dead
	{
		vec_set(my.x,you.x);
		my.pan = you.pan;
		my.tilt = barrel_offset;

		if (key_cuu == 1 && barrel_offset < 90) // use cursor up
		{
			barrel_offset += 3 * time;
		}
		if (key_cud == 1 && barrel_offset > 0) // and cursor down to change the speed
		{
			barrel_offset -= 3 * time;
		}

		vec_for_vertex(rocket_coords, my, 33); // vertex for player's rocket
		if (key_ctrl == 1 && rocket_ptr == null && lives > 0) // ctrl fires a rocket
		// if we press ctrl and the previous rocket has disappeared and the player isn't dead
		{
			ent_create (rocket_mdl, rocket_coords, fire_rocket);
			snd_play (rocket_wav, 100, 0);
		}

		if (you.invisible == off) // used for blinking when the player loses a life
		{
			my.invisible = off;
		}
		else
		{
			my.invisible = on;
		}

   		wait(1);
	}
	ent_remove(my);
}

function fire_rocket()
{
	while (rocket_ptr != null) {wait (1);} // disable auto fire
	rocket_ptr = me;
	wait (1);
	my.pan = you.pan;
	my.tilt = you.tilt;
	my.enable_impact = on;
	my.enable_block = on;
	my.enable_entity = on;
	my.event = rocket_explodes;
	while ((my != null) && (vec_dist(my.x, player.x) < 2000))
	{
		rocket_speed.x = 100;
		rocket_speed.y = 0;
		rocket_speed.z = 0;
		rocket_speed *= time; 
		move_mode = ignore_you + ignore_passable; // ignores the barrel -> can't collide with it
		ent_move (rocket_speed, nullvector);
		wait (1);
	}
	ent_remove (me);
}	

function rocket_explodes()
{
	wait (1);
	snd_play (explosion_wav, 40, 0);
	vec_set (temp, my.pos);
	temp.y -= 50; // move the explosion sprite a little closer to the player
	ent_create(explosion_pcx, temp, animated_explosion); 
	ent_remove (me); 
}

function animated_explosion()
{
	wait (1);
	my.passable = on;
	my.flare = on;
	my.bright = on;
	my.ambient = 100;
	while (my.frame < 7)
	{
		my.frame += 1 * time;
		wait (1);
	}
	ent_remove (me);
}

function vehicle_jumps()
{
	if (my.skill40 == 1) {return;} // disable multiple jumps
	snd_play (jump_wav, 30, 0);
	my.skill40 = 1; 
	while (my.z < 300)
	{
		vehicle_speed.z = 5 * time;
		wait (1);
	}
	while (my.z > 20)
	{
		vehicle_speed.z = -5 * time;
		wait (1);
	}
	vehicle_speed.z = 0;
	my.z = 20; // restore the exact vehicle height
	my.skill40 = 0; // allow a new jump
	snd_play (landed_wav, 30, 0);
}

action obstacle
{
	my.enable_impact = on;
	my.enable_entity = on;
	my.event = obstacle_hit;
}

function obstacle_hit()
{
	if (you != player) {return;}
	wait (1);
	my.passable = on; // don't trigger other events for this entity
	lives -= 1;
	player.invisible = on; // make the player blink 3 times
	sleep (0.3);
	snd_play (lost_wav, 60, 0);
	player.invisible = off;
	sleep (0.3);
	player.invisible = on;
	sleep (0.3);
	snd_play (lost_wav, 60, 0);
	player.invisible = off;
	sleep (0.3);
	player.invisible = on;
	sleep (0.3);
	snd_play (lost_wav, 60, 0);
	player.invisible = off;
}

action ufo
{
	while (player == null) {wait (1);}
	my.y = player.y; // we don't have to align the enemies in Wed
	my.enable_impact = on;
	my.enable_entity = on;
	my.event = destroy_ufo;
	while (vec_dist (player.x, my.x) > 1000) // if the player is far away
	{
		wait (1); // wait
	}
	while (my.x > player.x)
	{
		my.x -= 5 * time;
		if (total_frames % 90 == 1) // we have limited the frame rate to 30 fps, remember?
		{				
			ent_create (rocket_mdl, my.pos, fire_bomb); // fire a rocket every 3 seconds
		}
		wait (1);
	}
}		

function destroy_ufo()
{
	wait (1);
	score += 20;
	ent_remove (me);
}

function fire_bomb()
{
	wait (1);
	vec_set (temp.x, player.x); 
	vec_sub (temp.x, my.x);
	vec_to_angle (my.pan, temp); // rotate the bomb towards the player	
	my.enable_impact = on;
	my.enable_block = on;
	my.enable_entity = on;
	my.event = hit_player; 
	while (my != null)
	{
		bomb_speed.x = 100;
		bomb_speed.y = 0;
		bomb_speed.z = 0;
		bomb_speed *= time; 
		move_mode = ignore_you + ignore_passable; // ignores the barrel -> can't collide with it
		ent_move (bomb_speed, nullvector);
		wait (1);
	}
}	

function hit_player()
{
	wait (1);
	snd_play (explosion_wav, 40, 0);
	vec_set (temp, my.pos);
	temp.y -= 50; // move the explosion sprite a little closer to the player
	ent_create(explosion_pcx, temp, animated_explosion); 
	ent_remove (me); 
	lives -= 1;
}
